//
//  StudioUtilityPlugIn.swift
//  StudioUtility
//
//  Created by Marin Nagy on 17/06/2024.
//

@objc(StudioUtilityPlugIn) class StudioUtilityPlugIn : NSObject, FxTileableEffect
{
    
    let _apiManager : PROAPIAccessing!
        
    required init?(apiManager: PROAPIAccessing) {
        _apiManager = apiManager
    }
    
    func getParamValue(paramID: UInt32) -> Int32 {
        
        /// Retrieve and returns the value of the parameter
        
        let paramAPIRetrival = _apiManager!.api(for: FxParameterRetrievalAPI_v6.self) as! FxParameterRetrievalAPI_v6
        
        var tokenValue: Int32 = 0
        paramAPIRetrival.getIntValue(&tokenValue, fromParameter: paramID, at: CMTime.zero)
        
        return tokenValue
    }
    
    func setParamValue(paramID: UInt32) {
        
        /// Set the value of the parameter
        
        let paramAPISettings = _apiManager!.api(for: FxParameterSettingAPI_v5.self) as! FxParameterSettingAPI_v5
        
        let newToken: Int32 = 20 // new value for parameter
        paramAPISettings.setIntValue(newToken, toParameter: paramID, at: CMTime.zero)
        
    }
    
    func pluginInstanceAddedToDocument() {
                
        let currentValue: Int32 = getParamValue(paramID: 1)
        print(currentValue) // Works (got 10, wich is the initial parameter value)
        
        setParamValue(paramID: 1) // Got error: Swift runtime failure: Unexpectedly found nil while unwrapping an Optional value
    }
    
    func properties(_ properties: AutoreleasingUnsafeMutablePointer<NSDictionary>?) throws {
        let swiftProps = [
            kFxPropertyKey_MayRemapTime : NSNumber(booleanLiteral: false),
            kFxPropertyKey_PixelTransformSupport : NSNumber(value: kFxPixelTransform_Full),
            kFxPropertyKey_VariesWhenParamsAreStatic: NSNumber(booleanLiteral: false),
            kFxPropertyKey_ChangesOutputSize : NSNumber(booleanLiteral: false)
        ]
        let props = NSDictionary(dictionary: swiftProps)
        properties?.pointee = props
    }
    
    func addParameters() throws {
        
        let paramAPI = _apiManager!.api(for: FxParameterCreationAPI_v5.self) as! FxParameterCreationAPI_v5
        
        // Create the Int Slider parameter parameter
                
        paramAPI.addIntSlider(withName: "token", parameterID: 1, defaultValue: 10, parameterMin: 0, parameterMax: 999999, sliderMin: 0, sliderMax: 999999, delta: 1, parameterFlags: FxParameterFlags(kFxParameterFlag_DEFAULT | kFxParameterFlag_NOT_ANIMATABLE))
        
    }
    
    // FxTileableEffect
    
    func pluginState(_ pluginState: AutoreleasingUnsafeMutablePointer<NSData>?, at renderTime: CMTime, quality qualityLevel: UInt) throws {
        return
    }
    
    func destinationImageRect(_ destinationImageRect: UnsafeMutablePointer<FxRect>, sourceImages: [FxImageTile], destinationImage: FxImageTile, pluginState: Data?, at renderTime: CMTime) throws {
        destinationImageRect.pointee = sourceImages[0].imagePixelBounds
    }
    
    func sourceTileRect(_ sourceTileRect: UnsafeMutablePointer<FxRect>, sourceImageIndex: UInt, sourceImages: [FxImageTile], destinationTileRect: FxRect, destinationImage: FxImageTile, pluginState: Data?, at renderTime: CMTime) throws {
        sourceTileRect.pointee = destinationTileRect;
    }
    
    func renderDestinationImage(_ destinationImage: FxImageTile, sourceImages: [FxImageTile], pluginState: Data?, at renderTime: CMTime) throws {
        return
    }
    
}

